with Test_DoblDobl_Vector_Norms;

procedure ts_ddnorm is

-- DESCRIPTION :
--   Calls the main interactive test on vector norms
--   in double double precision.

begin
  Test_DoblDobl_Vector_Norms.Main;
end ts_ddnorm;
