with Test_mtAlgoDiff_Convolutions;

procedure ts_mtadcnv is

-- DESCRIPTION :
--   Calls the main test on algorithmic differentiation to evaluate and 
--   differentiate polynomial systems at truncated power series 
--   with multitasking.

begin
  Test_mtAlgoDiff_Convolutions.Main;
end ts_mtadcnv;
