with HexaDobl_Complex_Ring;
with Generic_Vectors;

package HexaDobl_Complex_Vectors is new Generic_Vectors(HexaDobl_Complex_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of hexa double complex numbers.
