with Test_Floating_Matrices;

procedure ts_fltmat is

-- DESCRIPTION :
--   Calls the main interactive test on matrices with double precision
--   and multiprecision floating-point numbers.

begin
  Test_Floating_Matrices.Main;
end ts_fltmat;
