with Standard_Complex_Poly_Ring;
with Generic_Vectors;

package Standard_Complex_Poly_Vectors is
  new Generic_Vectors(Standard_Complex_Poly_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of standard complex polynomials.
