with Generic_Polynomials;
with Quad_Double_Ring;

package Quad_Double_Polynomials is 
  new Generic_Polynomials(Quad_Double_Ring);

-- DESCRIPTION :
--   Defines polynomials over the ring of quad double numbers.
