with Generic_Polynomials;
with Standard_Floating_Ring;

package Standard_Floating_Polynomials is 
  new Generic_Polynomials(Standard_Floating_Ring);

-- DESCRIPTION :
--   Defines the polynomials over the ring of standard floating numbers.
