with DoblDobl_Dense_Series_Ring;
with Generic_Vectors;

package DoblDobl_Dense_Series_Vectors is 
  new Generic_Vectors(DoblDobl_Dense_Series_Ring);

-- DESCRIPTION :
--   Defines vectors over the ring of double double dense series.
